class Dset:
	
    vmap ={}

    def __init__(self, value):
        print("this is init")
        self.value = value
        self.parent = None
        self.children = []
        self.height = 0
        Dset.vmap[value]=self
        

    def union(self,other):
        #print("start find")
        selfroot = self.find()
        otheroot =other.find()
        #print("end find")
        if selfroot.height<otheroot.height:
            otheroot.children.append(selfroot)
            selfroot.parent = otheroot
        else:
            selfroot.children.append(otheroot)
            otheroot.parent = selfroot
            if selfroot.height == otheroot.height:
                selfroot.height+=1

    def __new__(cls,v):
        if v in Dset.vmap:
            return Dset.vmap[v]
        else:
            result = super().__new__(cls)
            return result
            
    def find(self):
        #print("self:",self)
        #print("self.parent:",self.parent)
        if self.parent is None:
            return self
        else:
            curp = self.parent
            #print("self:",self)
            #print("curp.children:",curp.children)
            curp.children.remove(self)
            while curp.parent is not None:
                curp = curp.parent
            curp.children.append(self)
            self.parent= curp
            return curp

    def __eq__(self,other):
        return self.value == other.value

    def tostring(self):
        s= str(self.value)
        if self.children ==[]:
            return s
        else:
            for child in self.children:
                s+=","+child.tostring()
            return s

    def __repr__(self):
        return "{" + self.tostring()+"}"




a= Dset("a")
b = Dset("b")























import string
from pprint import *

def inilist(n, s=None):
    return [s for i in range(n)]


def inilist2d(i, j, s=None):

    return [[s for x in range(j)] for y in range(i)]


def slist(s, sep=","):
    s = s.split(sep)
    return [v.strip() for v in s]


def isnum(s):

    try:
        s = int(s)
        if isinstance(s, int):
            print(s)
            return True
    except:
        pass
    try:
        s = float(s)
        if isinstance(s, float):
            return True
    except:
        pass
    return False


def valueof(s):
    if s == "True" or s == "False" or s == "None":
        return eval(s)
    if s == "":
        return None
    try:
        s = int(s)
        return s
    except:
        try:
            s = float(s)
            return s
        except:
            return s


def decomp1(s):
    l = s.split("=")
    for i in range(len(l)):
        l[i] = slist(l[i])
    return l


def decomp2(s):
    l = s.split(",")
    for i in range(len(l)):
        l[i] = slist(l[i], "=")
    t = [[], []]
    for v in l:
        t[0].append(v[0])
        t[1].append(v[1])
    return t


def decomp(s):
    if s.count("=") == 1:
        return decomp1(s)
    else:
        return decomp2(s)


def rotlist12(ls):
    return [[v] for v in ls]


def rotlist21(ls):
    result = []
    for v in ls:
        result += v
    return result


def wid(t):
    return len(t.array2d[0])


def strsep2list(s, sep='\n', limit=None):
    result = []
    i = 0
    n = len(s)
    sub = ""
    mlen = 0
    while i < n:
        if s[i] not in string.printable:
            mlen += 2
        else:
            mlen += 1
        sub +=s[i]
        if s[i] == '\n':
            result.append(sub[:-1])
            sub = ""
            mlen = 0
        elif s[i] == sep:
            result.append(sub)
            sub = ""
            mlen = 0
        elif mlen > limit:
            result.append(sub[:-1])
            sub = ""
            mlen = 0
            i-=1
        elif mlen == limit:
            result.append(sub)
            sub = ""
            mlen = 0
        elif i == len(s) - 1:
            result.append(sub)
            sub = ""
            mlen = 0
        else:
            pass

        i += 1
    #print("result = ")
    #pprint(result)
    return result



def clen(s):
    result = 0
    for i in range(len(s)):
        if s[i] not in string.printable:
            result += 2
        else:
            result += 1
    return result

def strsep2maxlen(s, sep='\n'):
    result = 0
    i = 0
    n = len(s)
    length = 0
    while i < n:
        if s[i] not in string.printable:
            adding = 2
        else:
            adding = 1
        length += adding
        if s[i] == '\n':
            sub = length - 1
            result = max(result, sub)
            length = 0
        elif s[i] == sep:
            sub = length
            result = max(result, sub)
            length = 0
        elif i == len(s) - 1:
            sub = length
            result = max(result,sub)
            length = 0
        i += 1
    return result


def maxlen(v, sep='\n'):
    if isinstance(v, str):
        return strsep2maxlen(v, sep)
    else:
        return len(str(v))


def boolof(s):
    booldict = {"True": True, "False": False}
    if s == "" or s == "None":
        return None
    else:
        try:
            return booldict[s]
        except:
            raise Exception("\'{}\' cannot cast to int".format(s))


def intof(s):
    if s == "" or s == "None":
        return None
    else:
        try:
            return int(s)
        except:
            raise Exception("\'{}\' cannot cast to int".format(s))


def floatof(s):
    if s == "" or s == "None":
        return None
    else:
        try:
            return float(s)
        except:
            raise Exception("\'{}\' cannot cast to int".format(s))


def strof(s):
    if s == "" or s == "None":
        return None
    else:
        return s


import logging
import math
import weakref
import numpy
import matplotlib.pyplot as _plt


"""
source:
https://gist.github.com/t20100/e5a9ba1196101e618883
"""


class MplInteraction(object):
    """Base class for class providing interaction to a matplotlib Figure."""

    def __init__(self, figure):
        """Initializer
        :param Figure figure: The matplotlib figure to attach the behavior to.
        """
        self._fig_ref = weakref.ref(figure)
        self._cids = []

    def __del__(self):
        self.disconnect()

    def _add_connection(self, event_name, callback):
        """Called to add a connection to an event of the figure
        :param str event_name: The matplotlib event name to connect to.
        :param callback: The callback to register to this event.
        """
        cid = self.figure.canvas.mpl_connect(event_name, callback)
        self._cids.append(cid)

    def disconnect(self):
        """Disconnect interaction from Figure."""
        if self._fig_ref is not None:
            figure = self._fig_ref()
            if figure is not None:
                for cid in self._cids:
                    figure.canvas.mpl_disconnect(cid)
            self._fig_ref = None

    @property
    def figure(self):
        """The Figure this interaction is connected to or
        None if not connected."""
        return self._fig_ref() if self._fig_ref is not None else None

    def _axes_to_update(self, event):
        """Returns two sets of Axes to update according to event.
        Takes care of multiple axes and shared axes.
        :param MouseEvent event: Matplotlib event to consider
        :return: Axes for which to update xlimits and ylimits
        :rtype: 2-tuple of set (xaxes, yaxes)
        """
        x_axes, y_axes = set(), set()

        # Go through all axes to enable zoom for multiple axes subplots
        for ax in self.figure.axes:
            if ax.contains(event)[0]:
                # For twin x axes, makes sure the zoom is applied once
                shared_x_axes = set(ax.get_shared_x_axes().get_siblings(ax))
                if x_axes.isdisjoint(shared_x_axes):
                    x_axes.add(ax)

                # For twin y axes, makes sure the zoom is applied once
                shared_y_axes = set(ax.get_shared_y_axes().get_siblings(ax))
                if y_axes.isdisjoint(shared_y_axes):
                    y_axes.add(ax)

        return x_axes, y_axes

    def _draw(self):
        """Conveninent method to redraw the figure"""
        self.figure.canvas.draw()


class ZoomOnWheel(MplInteraction):
    """Class providing zoom on wheel interaction to a matplotlib Figure.
    Supports subplots, twin Axes and log scales.
    """

    def __init__(self, figure=None, scale_factor=1.1):
        """Initializer
        :param Figure figure: The matplotlib figure to attach the behavior to.
        :param float scale_factor: The scale factor to apply on wheel event.
        """
        super(ZoomOnWheel, self).__init__(figure)
        self._add_connection('scroll_event', self._on_mouse_wheel)

        self.scale_factor = scale_factor

    @staticmethod
    def _zoom_range(begin, end, center, scale_factor, scale):
        """Compute a 1D range zoomed around center.
        :param float begin: The begin bound of the range.
        :param float end: The end bound of the range.
        :param float center: The center of the zoom (i.e., invariant point)
        :param float scale_factor: The scale factor to apply.
        :param str scale: The scale of the axis
        :return: The zoomed range (min, max)
        """
        if begin < end:
            min_, max_ = begin, end
        else:
            min_, max_ = end, begin

        if scale == 'linear':
            old_min, old_max = min_, max_
        elif scale == 'log':
            old_min = numpy.log10(min_ if min_ > 0. else numpy.nextafter(0, 1))
            center = numpy.log10(
                center if center > 0. else numpy.nextafter(0, 1))
            old_max = numpy.log10(max_) if max_ > 0. else 0.
        else:
            logging.warning(
                'Zoom on wheel not implemented for scale "%s"' % scale)
            return begin, end

        offset = (center - old_min) / (old_max - old_min)
        range_ = (old_max - old_min) / scale_factor
        new_min = center - offset * range_
        new_max = center + (1. - offset) * range_

        if scale == 'log':
            try:
                new_min, new_max = 10. ** float(new_min), 10. ** float(new_max)
            except OverflowError:  # Limit case
                new_min, new_max = min_, max_
            if new_min <= 0. or new_max <= 0.:  # Limit case
                new_min, new_max = min_, max_

        if begin < end:
            return new_min, new_max
        else:
            return new_max, new_min

    def _on_mouse_wheel(self, event):
        if event.step > 0:
            scale_factor = self.scale_factor
        else:
            scale_factor = 1. / self.scale_factor

        # Go through all axes to enable zoom for multiple axes subplots
        x_axes, y_axes = self._axes_to_update(event)

        for ax in x_axes:
            transform = ax.transData.inverted()
            xdata, ydata = transform.transform_point((event.x, event.y))

            xlim = ax.get_xlim()
            xlim = self._zoom_range(xlim[0], xlim[1],
                                    xdata, scale_factor,
                                    ax.get_xscale())
            ax.set_xlim(xlim)

        for ax in y_axes:
            ylim = ax.get_ylim()
            ylim = self._zoom_range(ylim[0], ylim[1],
                                    ydata, scale_factor,
                                    ax.get_yscale())
            ax.set_ylim(ylim)

        if x_axes or y_axes:
            self._draw()


class PanAndZoom(ZoomOnWheel):
    """Class providing pan & zoom interaction to a matplotlib Figure.
    Left button for pan, right button for zoom area and zoom on wheel.
    Support subplots, twin Axes and log scales.
    """

    def __init__(self, figure=None, scale_factor=1.1):
        """Initializer
        :param Figure figure: The matplotlib figure to attach the behavior to.
        :param float scale_factor: The scale factor to apply on wheel event.
        """
        super(PanAndZoom, self).__init__(figure, scale_factor)
        self._add_connection('button_press_event', self._on_mouse_press)
        self._add_connection('button_release_event', self._on_mouse_release)
        self._add_connection('motion_notify_event', self._on_mouse_motion)

        self._pressed_button = None  # To store active button
        self._axes = None  # To store x and y axes concerned by interaction
        self._event = None  # To store reference event during interaction

    @staticmethod
    def _pan_update_limits(ax, axis_id, event, last_event):
        """Compute limits with applied pan."""
        assert axis_id in (0, 1)
        if axis_id == 0:
            lim = ax.get_xlim()
            scale = ax.get_xscale()
        else:
            lim = ax.get_ylim()
            scale = ax.get_yscale()

        pixel_to_data = ax.transData.inverted()
        data = pixel_to_data.transform_point((event.x, event.y))
        last_data = pixel_to_data.transform_point((last_event.x, last_event.y))

        if scale == 'linear':
            delta = data[axis_id] - last_data[axis_id]
            new_lim = lim[0] - delta, lim[1] - delta
        elif scale == 'log':
            try:
                delta = math.log10(data[axis_id]) - \
                    math.log10(last_data[axis_id])
                new_lim = [pow(10., (math.log10(lim[0]) - delta)),
                           pow(10., (math.log10(lim[1]) - delta))]
            except (ValueError, OverflowError):
                new_lim = lim  # Keep previous limits
        else:
            logging.warning('Pan not implemented for scale "%s"' % scale)
            new_lim = lim
        return new_lim

    def _pan(self, event):
        if event.name == 'button_press_event':  # begin pan
            self._event = event

        elif event.name == 'button_release_event':  # end pan
            self._event = None

        elif event.name == 'motion_notify_event':  # pan
            if self._event is None:
                return

            if event.x != self._event.x:
                for ax in self._axes[0]:
                    xlim = self._pan_update_limits(ax, 0, event, self._event)
                    ax.set_xlim(xlim)

            if event.y != self._event.y:
                for ax in self._axes[1]:
                    ylim = self._pan_update_limits(ax, 1, event, self._event)
                    ax.set_ylim(ylim)

            if event.x != self._event.x or event.y != self._event.y:
                self._draw()

            self._event = event

    def _zoom_area(self, event):
        if event.name == 'button_press_event':  # begin drag
            self._event = event
            self._patch = _plt.Rectangle(
                xy=(event.xdata, event.ydata), width=0, height=0,
                fill=False, linewidth=1., linestyle='solid', color='black')
            self._event.inaxes.add_patch(self._patch)

        elif event.name == 'button_release_event':  # end drag
            self._patch.remove()
            del self._patch

            if (abs(event.x - self._event.x) < 3 or
                    abs(event.y - self._event.y) < 3):
                return  # No zoom when points are too close

            x_axes, y_axes = self._axes

            for ax in x_axes:
                pixel_to_data = ax.transData.inverted()
                begin_pt = pixel_to_data.transform_point((event.x, event.y))
                end_pt = pixel_to_data.transform_point(
                    (self._event.x, self._event.y))

                min_ = min(begin_pt[0], end_pt[0])
                max_ = max(begin_pt[0], end_pt[0])
                if not ax.xaxis_inverted():
                    ax.set_xlim(min_, max_)
                else:
                    ax.set_xlim(max_, min_)

            for ax in y_axes:
                pixel_to_data = ax.transData.inverted()
                begin_pt = pixel_to_data.transform_point((event.x, event.y))
                end_pt = pixel_to_data.transform_point(
                    (self._event.x, self._event.y))

                min_ = min(begin_pt[1], end_pt[1])
                max_ = max(begin_pt[1], end_pt[1])
                if not ax.yaxis_inverted():
                    ax.set_ylim(min_, max_)
                else:
                    ax.set_ylim(max_, min_)

            self._event = None

        elif event.name == 'motion_notify_event':  # drag
            if self._event is None:
                return

            if event.inaxes != self._event.inaxes:
                return  # Ignore event outside plot

            self._patch.set_width(event.xdata - self._event.xdata)
            self._patch.set_height(event.ydata - self._event.ydata)

        self._draw()

    def _on_mouse_press(self, event):
        if self._pressed_button is not None:
            return  # Discard event if a button is already pressed

        if event.button in (1, 3):  # Start
            x_axes, y_axes = self._axes_to_update(event)
            if x_axes or y_axes:
                self._axes = x_axes, y_axes
                self._pressed_button = event.button

                if self._pressed_button == 1:  # pan
                    self._pan(event)
                elif self._pressed_button == 3:  # zoom area
                    self._zoom_area(event)

    def _on_mouse_release(self, event):
        if self._pressed_button == event.button:
            if self._pressed_button == 1:  # pan
                self._pan(event)
            elif self._pressed_button == 3:  # zoom area
                self._zoom_area(event)
            self._pressed_button = None

    def _on_mouse_motion(self, event):
        if self._pressed_button == 1:  # pan
            self._pan(event)
        elif self._pressed_button == 3:  # zoom area
            self._zoom_area(event)


def figure_pz(*args, **kwargs):
    """matplotlib.pyplot.figure with pan and zoom interaction"""
    fig = _plt.figure(*args, **kwargs)
    fig.pan_zoom = PanAndZoom(fig)
    return fig





import json
import pprint
import time
import os

class Node(dict):
    def __new__(cls,data):
        if isinstance(data,list):
            return [Node(e) for e in data]
        if type(data) == dict:
            return super(Node, cls).__new__(cls)
        else:
            return data
        
    def __init__(self,data):
        res = {}
        for key in data:
            res[key] = Node(data[key])
        super().__init__(res)
        self.__dict__=self

    def __add__(self,other):
        n = self & other
        if n != {}:
            raise Exception(str(n))
        res = {}
        for key in self:
            res[key] = self[key]
        for key in other:
            res[key] = other[key]
        return Node(res)

    def __or__(self, other):
        xor = self ^ other # in self or in other
        n = self & other # in self and in other
        res = {}
        for key in xor:
            res[key] = xor[key]
        for key in n:
            res[key] = n[key]
        return Node(res)

    def __and__(self, other):
        res = {}
        a = self
        b = other
        if len(other)<len(self):
            a,b = other,self
        for key in a:
            if key in b:
                if a[key] != b[key]:
                    res[key] = {"0":a[key],"1":b[key]}
                else:
                    res[key] = a[key]
        return Node(res)

    def __xor__(self, other):
        res = {}
        for key in self:
            if not key in other:
                res[key] = self[key]
        for key in other:
            if not key in self:
                res[key] = other[key]
        return Node(res)

    def __sub__(self, other):
        res = {}
        for key in self:
            if not key in other:
                res[key] = self[key]
        return Node(res)

    union = __or__
    intersect = __and__
    minus = __sub__

class Root:

    path=""
    precision = 2
    encoding = "utf-8"

    def fsee(self):
        #see all the object
        pprint.pprint(self.__dict__)

    def see(self):
        d=self.__dict__.copy()
        d["body"]="......"
        pprint.pprint(d)

    def __init__(self,body,name=None,subpath=""):
        self.body  = Node(body)
        self.name = name
        self.subpath = subpath

    def __str__(self):
        heads = "Root<{}>:\n".format(self.name)
        return heads+pprint.pformat(self.body)

    def __repr__(self):
        return str(self)

    def select(self,keynames):
        if isinstance(keynames, list) or isinstance(keynames,tuple):
            cur = self.body
            for keyname in keynames:
                cur = cur[keyname]
            if isinstance(cur,dict) or isinstance(cur,list):
                return Root(cur.copy())
            else:
                return Root(cur)
        else:
            raise Exception("keynames must be list of string or int")


    def read(name,subpath=""):
        t1 = time.time()
        fname = Root.path + subpath + name + ".json"
        with open(fname, "r",encoding=Root.encoding) as read_file:
            k = json.load(read_file)
        t2 = time.time()
        res = Root(k,name,subpath)
        t3 = time.time()
        print("READ <{}> FROM {} takes {} + {}".format(name, fname, round(t2 - t1, Root.precision),round(t3 - t2, Root.precision)))
        return res

    def save(self,name=None,show=True,subpath =None):
        if name is None:
            if self.name is None:
                raise Exception("give a name for the node to save")
        else:
            if isinstance(name, str):
                self.name = name
            else:
                raise Exception("name should be string")

        if subpath:
            self.subpath = subpath

        t1 = time.time()

        fullpath = Root.path+self.subpath
        if (fullpath != "") and (not os.path.isdir(fullpath)):
            os.mkdir(fullpath)

        fname = Root.path + self.subpath + self.name+".json"
        with open(fname, "w",encoding=Root.encoding) as write_file:
            json.dump(self.body, write_file)

        t2 = time.time()
        if show:
            print("SAVE <{}> TO {} takes {}".format(self.name, fname,round(t2-t1,Root.precision)))
        return






from pprint import *
from .utility import*

class Keymap(dict):

    symbol= "*"

    def __init__(self):
        dict.__init__(self)
        self.table = None
        self.key = []
        self.map = {}
        self["key"] = self.key
        self["map"] = self.map

    def fixlenmap(self):
        for v in self.key:
            self.table.lenmap[self.table.colmap[v]]+=len(Keymap.symbol)

    def __del__(self):
        self.table.setlenmap()
        if self.table.groupmap is not None:
            self.table.groupmap.fixlenmap()

    def make(table,s):
        result = Keymap()
        if isinstance(s,str):
            s = slist(s)
        result.key += s
        result.table = table
        result.table._REatris(result.key)
        for i in range(1,len(result.table)+1):
            t= tuple(result.table.getlist(i,result.key))
            if not t in result.map:
                result.map[t] = i
            else:
                print("row",result.map[t])
                print("row",i)
                raise Exception("{} is not a key".format(result.key))
        if result.table.lenmap is not None:
            result.lenmap = result.table.lenmap.copy()
            result.fixlenmap()
        return result


    def trackaddrow(self,i):

        for index in range(i,len(self.table)+1):
            t = tuple(self.table.getlist(index, self.key))
            self.map[t] +=1

    def trackdelrow(self,i):

        t = tuple(self.table.getlist(i, self.key))
        del self.map[t]
        for index in range(i+1, len(self.table) + 1):
            t = tuple(self.table.getlist(index, self.key))
            self.map[t] -= 1


    def updatekey(self,oldkey, newkey):
        for i in range(len(self.key)):
            if self.key[i] == oldkey:
                self.key[i]=newkey



    def tracksetentry(self,i,j,value):

        attname = self.table.array2d[0][j]
        if self.table.array2d[i][j] == value:
            return
        if attname in self.key:
            #make a key tuple
            r = []
            for s in self.key:
                if s == attname:
                    r.append(value)
                else:
                    r.append(self.table.array2d[i][self.table.colmap[s]])
            newt = tuple(r)
            #check existence
            if newt in self.map:
                raise Exception("key duplicated")
            #start update
            else:
                #try to del old tuple
                oldt = tuple(self.table.getlist(i, self.key))
                if oldt in self.map:
                    del self.map[oldt]
                #assign new tuple to map
                self.map[newt] = i








        
        
    
